#ifndef _OACLAIM_H_
#define _OACLAIM_H_

#include "OAContract.h"
#include "OAEscrowMessage.h"
#include "OAMetadata.h"

//! \brief This class stores information to be sent to judge by client

class OAClaim {
	public:
		//! Constructor
		/*! Claim stores contract, client's counter, 
		 * warranty escrow message, receipt and metadata
		 * @param contract	the OAContract object which is sent to client by server
		 * @param counter 	counter kept by client
		 * @param w 		warranty message which is sent to client by server
		 * @param receipt 	server's signature on contract
		 * @param metadata 	OAMetadata object which includes last metadata, its proof and server's signature on this metadata
		 */
		OAClaim(OAContract* contract, int counter,
				OAEscrowMessage* w, string* receipt,
				OAMetadata* metadata)
			: contract(contract), counter(counter),
			  warrantyVEMessage(w), receipt(receipt),
			  metadata(metadata) {}
		
		//! Copy Constructor
		OAClaim(const OAClaim &o)
			: contract(o.contract), counter(o.counter),
			  warrantyVEMessage(o.warrantyVEMessage),
			  receipt(o.receipt), metadata(o.metadata) {}

		//getters
		/*! @return the counter that client claims to be server's */
		int getCounter() const {return counter;}
		/*! @return server's signature on contract */
		string* getReceipt() const {return receipt;}
		/*! @return warranty sent to client by server */
		OAEscrowMessage* getWarrantyVEMessage()	const {return warrantyVEMessage;}
		/*! @return metadata message which includes last metadata, its proof and server's signature on this metadata */
		OAMetadata* getMetadata() const {return metadata;}
		/*! @return contract between server and client */
		OAContract* getContract()const {return contract;}

	private:
		OAContract* contract;
		int counter;
		OAEscrowMessage* warrantyVEMessage;
		string* receipt;
		OAMetadata* metadata;
		
		//! Boost Serializer for OAClaim class
		friend class boost::serialization::access;
		template <class Archive> 
		void serialize(Archive& ar, const unsigned int ver) {
			ar  & auto_nvp(*contract)
				& auto_nvp(counter)
				& auto_nvp(*warrantyVEMessage)
				& auto_nvp(*receipt)
				& auto_nvp(*metadata);
		}
};

#endif

