#ifndef _OAJUDGE_CPP_
#define _OAJUDGE_CPP_

#include "Timer.h"
#include "OA.h"
#include "OAJudge.h"

// Check the information sent in claim
bool OAJudge::checkClaim(const OAClaim &claim) {
	bool ClaimOK =
		// check contract date
		   (claim.getContract()->getDue() + OA_CONTRACT_TIME_ELASTICITY > time(NULL))
		
		// check receipt
		&& Signature::verify(*claim.getContract()->getServersSignPK(), *claim.getReceipt(),
							saveString(*claim.getContract()), regularPK->hashAlg)
		// check metadata
		&& Signature::verify(*claim.getContract()->getServersSignPK(), *claim.getMetadata()->getSignature(),
							saveString(claim.getCounter())+(*claim.getMetadata()->getMetadata()),
							regularPK->hashAlg);
	
	if (!ClaimOK)
		return false;
	
	// keep following information
	clientsPK = claim.getContract()->getClientsSignPK();
	serversPK = claim.getContract()->getServersSignPK();
	counter_s = claim.getCounter();
	coinPrime = claim.getWarrantyVEMessage()->getCoinPrime();
	escrow = claim.getWarrantyVEMessage()->getEscrow();
	contract = claim.getContract();
	
	delete &claim;
	return true;
}

// Compare the counters to decide type of dispute
int OAJudge::compareCounters(int counter_c, const string &signature) {
	// check the counter
	if (!Signature::verify(*clientsPK, signature, saveString(counter_c), regularPK->hashAlg))
		return 0;
	
	// Compare counters
	if (counter_c > counter_s+1)
		return OA_CLIENT_PAYS;
	if (counter_c < counter_s)
		return OA_SERVER_PAYS;
	if (counter_c == counter_s+1) {
		// check if any updates left
		if (contract->getMaxUpdates() >= counter_c)
			return OA_DENIAL_OF_UPDATE;
		else
			return OA_CLIENT_PAYS;
	}
	//if (counter_c == counter_s)
		return OA_CORRUPTION;
}

// Try to update (in case of OA_DENIAL_OF_UPDATE)
OAUpdate* OAJudge::update(OAUpdate *update) {
	// check signature on proof
	if (!Signature::verify(*clientsPK, *update->getSignOnCtr(), saveString(counter_c), regularPK->hashAlg))
		return NULL;
	return update;
}

// Check the metadata (in case of OA_DENIAL_OF_UPDATE)
OAMetadata* OAJudge::checkMetadata(OAMetadata* mm) {
	if (!Signature::verify(*serversPK, *mm->getSignature(), saveString(counter_c+1)+(*mm->getMetadata()), regularPK->hashAlg))
		return NULL;
	return mm;
}

// Check proof (in case of OA_CORRUPTION)
bool OAJudge::checkProof(string &proof) {
	delete &proof;
	return false;
}

// Decrypt and send the endorsement to client
vector<ZZ>* OAJudge::serverPays() {
	// decrypt endorsement
	vector<ZZ>* endorsement = new vector<ZZ>(verifiableDecrypter->decrypt(escrow->getCiphertext(), saveString(*contract), hashAlg));
	// make sure the endorsement on the coin is valid
	if(!coinPrime.verifyEndorsement(*endorsement))
		throw CashException(CashException::CE_FE_ERROR,
				"[Arbiter::sellerResolveI] invalid endorsement");
	return endorsement;
}

#endif

