#ifndef _OAMETADATA_H_
#define _OAMETADATA_H_

#include <string>

/*! \brief This class is a wrapper for the metadata messages
 * sent in the Official Arbitration protocol
 */

class OAMetadata {
	public:
		//! Constructor
		/** 
		 * @param md 		metadata
		 * @param proof 	proof of metadata,
		 * @param s			server's signature on concatanation of counter and metadata (proof instead of metadata, if metadata is created for the first time)
		 */
		OAMetadata(const string &md, const string &p, string &s)
			: metadata(md), proof(p), signature(s) {}

		//! Copy Constructor
		OAMetadata(const OAMetadata &o)
			: metadata(o.metadata), proof(o.proof),
			  signature(o.signature) {}
	
		//getters
		/*! @return metadata */
		string* const getMetadata() 	{return &metadata;}
		/*! @return proof of metadata */
		string* const getProof() 		{return &proof;}
		/*! @return server's signature on concatanation of counter and metadata (proof instead of metadata, if this is the first metadata message sent) */
		string* const getSignature() 	{return &signature;}
		
		//setter (added for the case counter == 0)
		/*! Special purpose method, to be used at the setup phase for
		 * client to replace the signature on concatanation of counter and proof
		 * with the signature on concatanation of counter and metadata
		 * @param sign server's signature on concatanation of counter and metadata
		 * @return void
		 */
		void setSignature(string& sign) { signature = sign; } 
		
	private:
		string metadata;
		string proof;
		string signature;
		
		//! Boost Serializer for OAMetadata message
		friend class boost::serialization::access;
		template <class Archive> 
		void serialize(Archive& ar, const unsigned int ver) {
			ar & auto_nvp(metadata)
				& auto_nvp(proof)
				& auto_nvp(signature);
		}
};

#endif

