#ifndef _OAUPDATE_H_
#define _OAUPDATE_H_

#include <string>

/*! \brief This class is a wrapper for the update messages,
 * consisting of update and signature on counter,
 * sent in the Official Arbitration protocol
 */

class OAUpdate {
	public:
		//! Constructor
		/** 
		 * @param update 	update applied on file
		 * @param singOnCtr	client's signature on counter
		 */
		OAUpdate(string* const update, string* const signOnCtr)
			: update(update), signOnCtr(signOnCtr) {}
	
		/*! Destructor */
		~OAUpdate() { delete update; }
		
		//getters
		/*! @return update applied to file */
		string* const getUpdate() const {return update;}
		/*! @return client's signature on counter */
		string* const getSignOnCtr() const {return signOnCtr;}

	private:
		string* update;
		string* signOnCtr;
		
		//! Boost Serializer for OAUpdate class
		friend class boost::serialization::access;
		template <class Archive> 
		void serialize(Archive& ar, const unsigned int ver) {
			ar  & auto_nvp(*update);
			ar	& auto_nvp(*signOnCtr);
		}
};

#endif

